<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Entities;

use DateTime;

/**
 * Files entity
 */
final class Files {

	/**
	 * @var DateTime Timestamp
	 */
	private $timestamp;

	/**
	 * @var int Format version
	 */
	private $version;

	/**
	 * @var array<File> Array of files
	 */
	private $files;

	/**
	 * Constructor
	 * @param int $timestamp Timestamp
	 * @param int $version Format version
	 * @param array<File> $files Array of files
	 */
	public function __construct(int $timestamp, int $version, array $files) {
		$this->timestamp = (new DateTime())->setTimestamp($timestamp);
		$this->version = $version;
		$this->files = $files;
	}

	/**
	 * Creates a new files entity from an array
	 * @param array<string,mixed> $data JSON data in an array
	 * @return Files Files entity
	 */
	public static function fromArray(array $data): self {
		$files = [];
		foreach ($data['files'] as $file) {
			$files[] = File::fromArray($file);
		}
		return new static($data['timestamp'], $data['version'], $files);
	}

	/**
	 * Returns the timestamp
	 * @return DateTime Timestamp
	 */
	public function getTimestamp(): DateTime {
		return $this->timestamp;
	}

	/**
	 * Returns the format version
	 * @return int Format version
	 */
	public function getVersion(): int {
		return $this->version;
	}

	/**
	 * Returns the array of files
	 * @return array<File> Array of files
	 */
	public function getFiles(): array {
		return $this->files;
	}

}
