<?php

/**
 * Copyright 2019 IQRF Tech s.r.o.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
declare(strict_types = 1);

namespace Iqrf\Repository\Models;

use Iqrf\Repository\Entities\Manufacturer;
use Iqrf\Repository\Exceptions\CorruptedData;
use Iqrf\Repository\Exceptions\DataNotFound;
use Iqrf\Repository\Exceptions\ManufacturerNotFound;
use Iqrf\Repository\Exceptions\ServiceUnavailable;

/**
 * Manufacturer manager
 */
final class ManufacturerManager extends BaseManager {

	/**
	 * Manufacturers path
	 */
	private const PATH = 'manufacturers';

	/**
	 * Lists all manufacturers
	 * @return array<Manufacturer> All manufacturers
	 * @throws CorruptedData
	 * @throws ServiceUnavailable
	 */
	public function list(): array {
		$response = $this->apiClient->sendRequest(self::PATH);
		foreach ($response as &$item) {
			$item = Manufacturer::fromArray($item);
		}
		return $response;
	}

	/**
	 * Returns the manufacturer by its ID
	 * @param int $id Manufacturer
	 * @return Manufacturer Manufacturer
	 * @throws CorruptedData
	 * @throws ManufacturerNotFound
	 * @throws ServiceUnavailable
	 */
	public function get(int $id): Manufacturer {
		try {
			$response = $this->apiClient->sendRequest(self::PATH . '/' . $id);
		} catch (DataNotFound $e) {
			throw new ManufacturerNotFound();
		}
		return Manufacturer::fromArray($response);
	}

}
